/* Copyright (C) 2014 by Bundesamt für Sicherheit in der Informationstechnik
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU GPL (v>=2)
 * and comes with ABSOLUTELY NO WARRANTY!
 * See LICENSE.txt for details.
 */
#include "certificatelistwidget.h"
#include <QDebug>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QLabel>
#include <QApplication>

#include "certificateitemwidget.h"

CertificateListWidget::CertificateListWidget(QWidget *parent, Qt::WindowFlags flags) :
    QWidget(parent, flags)
{
    QVBoxLayout *outerLayout =  new QVBoxLayout(this);
    outerLayout->addLayout(&mLayout);
    outerLayout->addStretch();
}

void CertificateListWidget::addCertificate(
    const Certificate &certificate,
    bool state,
    QToolButton *button)
{
    CertificateItemWidget *widget =
        new CertificateItemWidget(this, certificate, state, button);
    connect(widget, SIGNAL(stateChanged(bool, const Certificate&)),
        this, SLOT(certStateChanged(bool, const Certificate&)));

    mCertificateWidgets << widget;
    mLayout.addWidget(widget);
    emit certListChanged(-1);
}

int CertificateListWidget::activeCertificates()
{
    int ret = 0;
    for (int i = 0; i < mCertificateWidgets.size(); i++) {
        if (mCertificateWidgets[i]->certificate()->isActive()) {
            ret++;
        }
    }
    return ret;
}

void CertificateListWidget::activateCertificate(const Certificate &cert)
{
    for (int i = 0; i < mCertificateWidgets.size(); i++) {
        if (*(mCertificateWidgets[i]->certificate()) == cert) {
            mCertificateWidgets[i]->certificate()->setActive(true);
            break;
        }
    }
    emit certListChanged(-1);
}

void CertificateListWidget::deactivateCertificate(const Certificate &cert)
{
    for (int i = 0; i < mCertificateWidgets.size(); i++) {
        if (*(mCertificateWidgets[i]->certificate()) == cert) {
            mCertificateWidgets[i]->certificate()->setActive(false);
            break;
        }
    }
    emit certListChanged(-1);
}

void CertificateListWidget::removeCertificate(const Certificate &cert)
{
    for (int i = 0; i < mCertificateWidgets.size(); i++) {
        if (*(mCertificateWidgets[i]->certificate()) == cert) {
            mLayout.removeWidget(mCertificateWidgets[i]);
            mCertificateWidgets[i]->deleteLater();
            mCertificateWidgets.removeAt(i);
            break;
        }
    }
    emit certListChanged(-1);
}

void CertificateListWidget::clear()
{
    foreach (CertificateItemWidget * item, mCertificateWidgets) {
        mLayout.removeWidget(item);
        delete item;
    }
    mCertificateWidgets.clear();
}

QStringList CertificateListWidget::certificates()
{
    QStringList list;
    foreach (CertificateItemWidget * item, mCertificateWidgets) {
        list <<  item->certificate()->base64Line();
    }
    return list;
}

QStringList CertificateListWidget::selectedCertificates() {
    QStringList list;
    foreach (CertificateItemWidget * item, mCertificateWidgets) {
        if (item->state()) {
            list << item->certificate()->base64Line();
        }
    }
    return list;
}

QStringList CertificateListWidget::unselectedCertificates() {
    QStringList list;
    foreach (CertificateItemWidget * item, mCertificateWidgets) {
        if (!item->state()) {
            list << item->certificate()->base64Line();
        }
    }
    return list;
}

QList<Certificate> CertificateListWidget::certificateList()
{
    QList<Certificate> list;
    foreach (CertificateItemWidget * item, mCertificateWidgets) {
        list <<  *(item->certificate());
    }
    return list;
}

void CertificateListWidget::setCertState(bool state, const Certificate &cert)
{
    foreach (CertificateItemWidget * item, mCertificateWidgets) {
        if (*(item->certificate()) == cert &&
            item->state() != state) {
            item->setState(state);
        }
    }
}


void CertificateListWidget::certStateChanged(bool state, const Certificate &cert)
{
    emit certListChanged(-1);
    emit certChanged(state, cert);
}

int CertificateListWidget::selectedCertCount()
{
    int selected = 0;
    foreach (CertificateItemWidget * item, mCertificateWidgets) {
        if (item->state()) {
            selected++;
        }
    }
    return selected;
}

bool CertificateListWidget::contains(const Certificate &cert)
{
    foreach (CertificateItemWidget * item, mCertificateWidgets) {
        if (*(item->certificate()) == cert) {
            return true;
        }
    }
    return false;
}
